@@ -46,8 +46,10 @@ function Get(options) {
         }
         this.uri = options.uri;
         this.max_redirs = options.max_redirs || 10;
+        this.max_length = options.max_length || 0;
         this.encoding = options.encoding;
         this.headers = options.headers || default_headers;
+        this.timeout = 'timeout' in options ? options.timeout : 10000;
         if (!this.no_proxy && process.env.HTTP_PROXY) {
             this.proxy = url.parse(process.env.HTTP_PROXY);
         } else {
@@ -96,7 +98,7 @@ Get.prototype.request = function(callback) {
                  (this.uri_o.hash || '')) || '/'
         }, callback);
     }
-};
+};  
 
 
 // Innermost API function of Get
@@ -141,11 +143,25 @@ Get.prototype.perform = function(callback, times) {
 
     // The client can fail if the url is invalid
     if (clientrequest) {
+        // Ensure the callback is only called once in error cases.
+        // Timeouts can trigger both error and timeout callbacks.
+        var error = 0;
+
         // Handle DNS-level errors, like ECONNREFUSED
         clientrequest.on('error', function(err) {
+            if (++error > 1) return;
             return callback.call(this, err);
         }.bind(this));
 
+        // Enforce a timeout of 10 seconds.
+        // Add a no-op version of setTimeout for node <= 0.4.x.
+        clientrequest.setTimeout = clientrequest.setTimeout || function() {};
+        clientrequest.setTimeout(this.timeout, function() {
+            clientrequest.connection.end();
+            if (++error > 1) return;
+            return callback.call(this, new Error('Timed out after ' + this.timeout + 'ms'));
+        }.bind(this));
+
         // TODO: fix when/if gzip is supported.
         // If a proxy is defined, ask for the full requested URL,
         // otherwise construct the URL without a hostname and protocol.
@@ -191,35 +207,51 @@ Get.prototype.toDisk = function(filename, callback) {
 //
 // - @param {Function} callback.
 Get.prototype.asString = function(callback) {
+    var max_length = this.max_length;
+    var payload = 0;
+
     // TODO: catch all errors
     this.perform(function pipeResponseToString(err, response) {
         if (err) return callback(err);
 
-        switch ((response.headers['content-type'] || '').toLowerCase()) {
-            case 'binary/octet-stream':
-            case 'application/octet-stream':
+        switch ((response.headers['content-type'] || '').toLowerCase().split('/')[0]) {
+            case 'binary':
+            case 'application':
+            case 'image':
+            case 'video':
                 return callback(new Error("Can't download binary file as string"));
             default:
                 // TODO: respect Content-Transfer-Encoding header
                 response.setEncoding(this.guessEncoding(this.uri));
         }
 
+        function returnString() {
+            if (!callback) return;
+            callback(null, out.join(''), response.headers);
+            callback = null;
+        }
+
         // Fill an array with chunks of data,
         // and then join it into a string before calling `callback`
         var out = [];
         response.on('data', function(chunk) {
-            out.push(chunk);
+            if (!callback) return;
+            payload += chunk.length;
+            if (max_length && payload > max_length) {
+                response.socket.end();
+                callback(new Error('File exceeds maximum allowed length of ' + max_length + ' bytes'));
+                callback = null;
+            } else {
+                out.push(chunk);
+            }
         });
         response.on('error', function(err) {
-            return callback(err);
-        });
-        response.on('end', function() {
-            return callback(null, out.join(''), response.headers);
-        });
-        response.on('close', function() {
-            // TODO: should this be this way?
-            return callback(null, out.join(''), response.headers);
+            if (!callback) return;
+            callback(err);
+            callback = null;
         });
+        response.on('end', returnString);
+        response.on('close', returnString);
     });
 };
 
@@ -227,10 +259,14 @@ Get.prototype.asString = function(callback) {
 //
 // - @param {Function} callback.
 Get.prototype.asBuffer = function(callback) {
+    var max_length = this.max_length;
+    var payload = 0;
+
     this.perform(function(err, response) {
         if (err) return callback(err);
 
         function returnBuffer() {
+            if (!callback) return;
             for (var length = 0, i = 0; i < out.length; i++) {
                 length += out[i].length;
             }
@@ -239,17 +275,28 @@ Get.prototype.asBuffer = function(callback) {
                 out[j].copy(result, pos);
                 pos += out[j].length;
             }
-            return callback(null, result, response.headers);
+            callback(null, result, response.headers);
+            callback = null;
         }
 
         // Fill an array with chunks of data,
         // and then join it into a buffer before calling `callback`
         var out = [];
         response.on('data', function(chunk) {
-            out.push(chunk);
+            if (!callback) return;
+            payload += chunk.length;
+            if (max_length && payload > max_length) {
+                response.socket.end();
+                callback(new Error('File exceeds maximum allowed length of ' + max_length + ' bytes'));
+                callback = null;
+            } else {
+                out.push(chunk);
+            }
         });
         response.on('error', function(err) {
-            return callback(err);
+            if (!callback) return;
+            callback(err);
+            callback = null;
         });
         response.on('end', returnBuffer);
         response.on('close', returnBuffer);
